/*
 * Copyright (C) 2009 Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

var EXPORTED_SYMBOLS = ["AsyncRequest"];
Components.utils.import("resource://hultmann-migre-me-modules/util.js");

function AsyncRequest() {
  this.requestUrl = null;
  this.onError = null;
  this.onSuccess = null;
  this.onComplete = null;
}

AsyncRequest.prototype = {

  _createXhr: function() {
    var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);

    xhr.open("GET", this.requestUrl, true);
    xhr.setRequestHeader("Accept-Language", null);
    xhr.setRequestHeader("Cookie", null);
    xhr.setRequestHeader("Accept", "text/xml");
    xhr.setRequestHeader("User-Agent", 'SubLink/3.0.0 (+http://br.mozdev.org/migre.me/)');
    // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
    //xhr.channel.loadFlags |= Ci.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;
    return xhr;
  },


  convert: function(uri) {
    //this.requestUrl = "http://localhost:81/migre.me/api-test.html?url=" + encodeURIComponent(uri.spec);
    this.requestUrl = "http://migre.me/api.xml?url=" + encodeURIComponent(uri.spec);

    var xhr = this._createXhr();
    var me = this;
    xhr.onload = function(evt) {
      me.onComplete();
      me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
    }
    xhr.onerror = function(evt) {
      me.onComplete();
      me.requestFail(this);
    }
    xhr.send(null);
  },


  findRedir: function(uri) {
    var idx = uri.path.indexOf('/', 1);
    if (idx > -1) {
      if (uri.path.indexOf('/', idx + 1) == -1) {
        var uri = uri.clone();
        uri.path = uri.path.substr(idx);
      }
    }


    this.requestUrl = "http://migre.me/api_redirect2.xml?url=" + encodeURIComponent(uri.spec);
    var xhr = this._createXhr();
    var me = this;
    xhr.onload = function(evt) {
      me.onComplete();
      me.requestSuccess(this); // this == evt.target == evt.currentTarget == xhr;
    }
    xhr.onerror = function(evt) {
      me.onComplete();
      me.requestFail(this);
    }
    xhr.send(null);
  },


  requestFail: function(xhr) {
    var errorData = {
      id: "noconnection",
      xhr: xhr,
      msg: null
    };
    this.onError(errorData);
  },


  requestSuccess: function(xhr) {
    var errorData = {
      id: "noconnection",
      xhr: xhr,
      msg: null
    };

    if (xhr.status !== 200) {
      errorData.id = "invalid";
      this.onError(errorData);
      return;
    }

    var doc = xhr.responseXML;
    if (doc === null) {
      errorData.id = "invalid";
      this.onError(errorData);
      return;
    }


    if (doc.documentElement.namespaceURI === "http://www.mozilla.org/newlayout/xml/parsererror.xml") {
      errorData.id = "malformed";
      this.onError(errorData);
      return;
    }


    var responseData = this.populateResponseData(doc);
    responseData.responseStatusCode = 301;


    if (this.requestUrl.indexOf("http://migre.me/api.xml") !== 0) {

      if (this.validateRedirData(responseData, errorData)) {
        var uri = util.stringToUri(responseData.longUrl);
        if (uri === null) {
          this.onError({
            id: "invalid",
            xhr: xhr,
            msg: null
          });
        } else {
          responseData.longUri = uri;
          this.onSuccess(responseData);
        }
      } else {
        this.onError(errorData);
      }

    } else {

      if (this.validateCompressResponse(responseData, errorData)) {
        var uri = util.stringToUri(responseData.shortUrl);
        if (uri === null) {
          this.onError({
            id: "invalid",
            xhr: xhr,
            msg: null
          });
        } else {
          this.onSuccess({
            shortUri:           uri,
            responseStatusCode: xhr.status
          });
        }
      } else {
        this.onError(errorData);
      }
    }
  },


  /*
  <?xml version="1.0" encoding="iso-8859-1"?>
  <item>
    <title>XML Migre.me</title>
    <error>0</error>
    <errormessage>No errors</errormessage>
    <created_at>Tue, 20 Oct 2009 01:23:51 -0200</created_at>
    <id>9sw6</id>
    <migre>http://migre.me/9sw6</migre>
    <url><![CDATA[http://desciclo.pedia.ws/wiki/Firefox]]></url>
    <title><![CDATA[Microsoft Volkswagem Mozilla Firefox - Desciclopédia]]></title>
    <content_type><![CDATA[text/html]]></content_type>
  </item>
  */

  populateResponseData: function(doc) {
    var rv = {
      errorCode: null,
      errorMessage: null,
      shortUrl: null,

      longUrl: null,
      contentType: null,
      htmlTitle: null
    };

    //var nodeSelectors = ["item > error", "item > errormessage", "item > migre", "item > url", "item > content_type", "item > title"];
    var xmlObj = ["/item/error", "/item/errormessage", "/item/migre", "/item/url", "/item/content_type", "/item/title"];
    var rvObj  = [  "errorCode",       "errorMessage",    "shortUrl",   "longUrl",        "contentType",   "htmlTitle"];


    var type = Ci.nsIDOMXPathResult.ANY_TYPE;
    var xmlValues = xmlObj.map(function(query) {
      return doc.evaluate(query, doc, null, type, null).iterateNext();
    });

    rvObj.forEach(function(val, idx) {
      if (xmlValues[idx] !== null) {
        rv[val] = xmlValues[idx].textContent;
      }
    });

  /*
    rvObj.forEach(function(val, idx) {
      var node = doc.querySelector(nodeSelectors[idx]);
      if (node) {
        rv[val] = node.textContent;//nodeValue
      }
    });
  */

    return rv;
  },


  validateCompressResponse: function(responseData, errorData) {
    if (responseData.errorCode === null) {
      errorData.id = "invalid";
      return false;
    }
    if (responseData.errorCode !== "0") {
      errorData.id = "couldntconvert";
      errorData.msg = responseData.errorMessage === null ? "" : responseData.errorMessage;
      return false;
    }

    if (responseData.shortUrl === null) {
      errorData.id = "invalid";
      return false;
    }

    return true;
  },


  validateRedirData: function(responseData, errorData) {
    if (responseData.errorCode === null) {
      errorData.id = "invalid";
      return false;
    }
    if (responseData.errorCode !== "0") {
      errorData.id = "couldntrestore";
      errorData.msg = responseData.errorMessage === null ? "" : responseData.errorMessage;
      return false;
    }

    if (responseData.longUrl === null) {
      errorData.id = "invalid";
      return false;
    }

    return true;
  }

};
