/*
 * Copyright (C) 2009 Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

var EXPORTED_SYMBOLS = ["notificationError", "util3"];
Components.utils.import("resource://hultmann-migre-me-modules/util.js");

function notificationError(errData, browserOrigin) {
util.log("notificationError: " + errData.toSource() + errData.xhr.getResponseHeader("Location"));

  var url = "URL_UNKNOWN";
  var host = "HOST_UNKNOWN";
  var method = "METHOD_UNKNOWN";

  if (errData.xhr.channel) {//host not found?
    var uri = errData.xhr.channel.originalURI;
    method = errData.xhr.channel.requestMethod;
    url = uri.spec;
    host = uri.host;
  }

  var buildLog = function() {
    var x = errData.xhr;
    return method + " " + url + "\n====================8<-----------------\n" +
           x.status + " " + x.statusText + "\n" +
           x.getAllResponseHeaders() + "\n" +
           x.responseText;
  };

  var box = util3.getNotification(browserOrigin);
  var label = util.getText("notification.response.label", host);

  switch (errData.id) {
    case "couldntconvert":
      showNotification(box, util.getText("notification.request.compress.label", host, errData.msg), null, null);
      break;
    case "couldntrestore-short":
      showNotification(box, util.getText("notification.request.uncompressShort.label", host), null, null);
      break;
    case "couldntrestore":
      showNotification(box, util.getText("notification.request.uncompress.label", host, errData.msg), null, null);
      break;
    case 'invalid':
      showNotification(box, label,       util.getText("error.response.details.invalid", host), buildLog());
      break;
    case 'malformed':
      showNotification(box, label,       util.getText("error.response.details.malformed", host), buildLog());
      break;
    case 'noconnection':
      showNotification(box, util.getText("notification.connection.label", host), null, null);
      break;
    //case 1:
    //  showNotification(box, label, util.getText('error.response.details.status'), buildLog());
    //  break;
    //case 'nodata':
    //  showNotification(box, label,       util.getText('error.response.details.nodata'), buildLog());
    //  break;
    default:
      throw util.myThrow("BUG");
  }
}



function showNotification(box, msg, error, txtLog) {
  var chromeWnd = box.ownerDocument.defaultView;

  var buttonConfig = null;
  if (error !== null) {
    buttonConfig = [{
      callback: function(evt) {
        var params = {
          title: util.getText('error.response.details.title'),
          close: util.getText('error.response.details.close.label'),
          label: error,
          description: txtLog
        };
        chromeWnd.openDialog('chrome://hultmann-migre-me-content/content/warning.xul',
                          '',
                          'chrome,centerscreen,modal',
                          params);
        return true;
      },
      label: util.getText('notification.button.label'),
      accessKey: util.getText('notification.button.accesskey')
    }];
  }

  var g = chromeWnd.getBrowser();
  var box = g.getNotificationBox(g.selectedBrowser);
  var notif = box.appendNotification(
                  msg,
                  'hultmann-migre-me-content-warning',
                  'chrome://global/skin/icons/warning-16.png',
                  box.PRIORITY_WARNING_MEDIUM,
                  buttonConfig);
}



const util3 = {

  getNotification: function(browserOrigin) {
    var tabBrowser = browserOrigin.ownerDocument.defaultView.getBrowser();
    var sel = this._findTabByBrowser(browserOrigin, tabBrowser);
    if (sel !== null) {
      tabBrowser.selectedTab = sel;
    } else {
      tabBrowser = Cc["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Ci.nsIWindowMediator)
                    .getMostRecentWindow("navigator:browser")
                    .getBrowser();
      browserOrigin = tabBrowser.selectedBrowser;
    }
    return tabBrowser.getNotificationBox(browserOrigin);
  },

  _findTabByBrowser: function(browser, tabBrowser) {
      const nodes = tabBrowser.mTabContainer.childNodes;
      for (var idx = 0, len = nodes.length; idx < len; idx++) {
          if (nodes[idx].linkedBrowser === browser) {
              return nodes[idx];
          }
      }
      return null;
  },

  showNotFoundNotification: function(box, newUrl) {
    var buttonConfig = [{
      callback: function(evt) {
        util.toClipboard(newUrl);
        return false;
      },
      label: util.getText("notification.node.notfound.button.label"),
      accessKey: util.getText("notification.node.notfound.button.accesskey")
    }];

    var notif = box.appendNotification(
                    util.getText("notification.node.notfound.label", newUrl),
                    "hultmann-migre-me-content-warning",
                    "chrome://global/skin/icons/warning-16.png",
                    box.PRIORITY_WARNING_MEDIUM,
                    buttonConfig);
  }
};
