/*
 * Copyright (C) 2009 Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

const EXPORTED_SYMBOLS = ["cmdPopupshowing", "cmdPopupshown"];
const NS_XUL = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

Components.utils.import("resource://hultmann-migre-me-modules/util.js");


function cmdPopupshowing(mainMenu, myTarget) {
  var chromeDoc = mainMenu.ownerDocument;

  if (!chromeDoc.popupNode) {
    util.log("chromeDoc.popupNode=" + chromeDoc.popupNode);
    return;
  }


  if (isTextInputNode(chromeDoc.popupNode)) {
    // makeMenuInputText
    // text input?
    makeMenuInputText(mainMenu, myTarget);

  } else {
    // link/document
    if (myTarget.id === "contentAreaContextMenu") {
      util.log("gContextMenu");
      makeMenuDocument(mainMenu, chromeDoc.defaultView.gContextMenu.linkURL);
    }
  }

}


function cmdPopupshown(mainMenu) {
  var clickedNode = mainMenu.ownerDocument.popupNode;

  if (!clickedNode) {
    util.log("clickedNode=" + clickedNode );
    return;
  }

  if (clickedNode.namespaceURI !== NS_XUL) {
    return;
  }

  if (mainMenu.querySelector(".hultmann-migre-me-node") !== null) {
    util.log("menu present!");
    return;
  }


  // chrome menu?
  function getAnchor(clickedNode2) {
    var findAnchor = function(elem) {
      if (elem === null) {
        return null;
      }
      var node = elem;
      while (node.tagName) {
        //util.log('this node '+node.tagName + (typeof node) + node.toString() + node.toSource() + (el instanceof Components.interfaces.nsIDOMXULElement));
        //util.log("this node "+node.tagName+' href='+node.href);
        //if (node.tagName === 'A') {
        //if (node instanceof Components.interfaces.nsIDOMHTMLAnchorElement && node.href) {
        if (node.href) {
          util.log("found");
          return node.href;
        }
        if (node.hasAttribute("href")) {
          //util.log("href found");
          return node.getAttribute("href");
        }
        node = node.parentNode;
      }
      return null;
    };
    return findAnchor(clickedNode2);
  }


  var url = getAnchor(clickedNode); // clickedNode may be invalid
  makeMenuDocument(mainMenu, url);
}


function makeMenuInputText(mainMenu, myTarget) {
  if (isTextMenu(mainMenu) === false) {
    return;
  }

  var chromeDoc = mainMenu.ownerDocument;

  var addr = findClickedWord(chromeDoc);
  if (addr === null) {
    return;
  }

  var uri = toUri(addr);
  if (uri === null) {
    return;
  }


  mainMenu.addEventListener("popuphidden", onPopupHidden, false);

  var isUrlbar = myTarget.id === "urlbar";
  var cmdData = {
    uri: uri,
    typed: addr,
    node: chromeDoc.popupNode,
    selectionStart: chromeDoc.popupNode.selectionStart,
    selectionEnd: chromeDoc.popupNode.selectionEnd
  };

  if (util.isRedirUri(cmdData.uri)) {
    Components.utils.import("resource://hultmann-migre-me-modules/menu-resolve.js");
    buildResolveMenu(mainMenu, cmdData, isUrlbar);
  } else {
    Components.utils.import("resource://hultmann-migre-me-modules/menu-create.js");
    buildCreateRedirMenu(mainMenu, cmdData, isUrlbar);
  }
}


function makeMenuDocument(mainMenu, linkUrl) {

  var useSel = false;
  var sel = new Selection(mainMenu.ownerDocument);
  if (sel.hasSelection) {
    var s = sel.getValidSelection();
    if (s !== null) {
      useSel = true;
    }
  }


  if (linkUrl && !useSel) {
    // link
    if (makeLinkMenuCore(mainMenu, linkUrl) === false) {
      var node = mainMenu.ownerDocument.popupNode;
      util.log('node ns='+  node.namespaceURI);
      var isNodeAnchor = false;
      switch (node.namespaceURI) {
        case NS_XUL:
        case "http://www.w3.org/1999/xhtml":
          isNodeAnchor = node.tagName === "a";
          break;
        default:
          isNodeAnchor = node.tagName === "A";
          break;
      }
      if (isNodeAnchor) {
        //util.log("linkUrl " + node.textContent + util.stringToUri(node.textContent).spec);
        makeLinkMenuCore(mainMenu, node.textContent);
      }
    }

  } else {
    // document
    var addr = findClickedWord(mainMenu.ownerDocument);
    util.log("word:"+addr);
    makeLinkMenuCore(mainMenu, addr);
  }

}


function makeLinkMenuCore(mainMenu, url) {
  util.log('makeLinkMenuCore='+url);

  if (url === null) {
    util.log('url=null');
    return false;
  }

  var redir = toUri(url);
  if (redir === null) {
    return false;
  }


  if (util.isRedirUri(redir) === false) {
    return false;
  }

  mainMenu.addEventListener("popuphidden", onPopupHidden, false);

  // migre.me, is.gd, bit.ly, ...

  var cmdData = {
    uri: redir,
    typed: null,
    node: null,
    selectionStart: 0,
    selectionEnd: 0
  };

  Components.utils.import("resource://hultmann-migre-me-modules/menu-resolve.js");
  buildResolveMenu(mainMenu, cmdData, false);
  return true;
}


function onPopupHidden(evt) {
  var mainMenu = evt.originalTarget;
  var doc = mainMenu.ownerDocument;
  mainMenu.removeEventListener("popuphidden", onPopupHidden, false);

  //var nodeList = mainMenu.getElementsByClassName("hultmann-migre-me-node");//get some "undefined" elements o_O
  var nodeList = mainMenu.querySelectorAll(".hultmann-migre-me-node");
  for (var idx = 0, len = nodeList.length; idx < len; idx++) {
    var node = nodeList[idx];
    //util.log("onPopupHidden del " + node.id);
    node.parentNode.removeChild(node);
  }

}





function Selection(chromeDoc) {
  this._chromeDoc = chromeDoc;
  this._input = isTextInputNode(chromeDoc.popupNode);
}

Selection.prototype = {

  get hasSelection() {
    var clickedNode = this._chromeDoc.popupNode;
    if (this._input) {
      return clickedNode.selectionStart !== clickedNode.selectionEnd;
    } else {
      var contentDoc = clickedNode.ownerDocument;
      var sel = contentDoc.defaultView.getSelection();
      return (sel.rangeCount === 1) && (sel.toString().length > 0);
    }
  },

  _getFromInput: function() {
    var clickedNode = this._chromeDoc.popupNode;

    // selectionStart/selectionEnd appear to be defined only on input nodes
    return clickedNode.value.substring(clickedNode.selectionStart,
                                       clickedNode.selectionEnd);
  },

  _getFromDoc: function() {
    var clickedNode = this._chromeDoc.popupNode;

    var contentDoc = clickedNode.ownerDocument;
    var sel = contentDoc.defaultView.getSelection();
    util.log("sel2:"+sel);

    if (sel.rangeCount !== 1) {
      return null;
    }
    util.log("rangeCount:"+sel.rangeCount);
    if (sel.containsNode(clickedNode, true) === false) {
      return null;
    }

    util.log("containsNode:ok "+sel.toString() + (typeof sel.toString()));
    return sel.toString();
  },

  getValidSelection: function() {
    var clickedNode = this._chromeDoc.popupNode;
    var sel;
    if (isTextInputNode(clickedNode)) {
      sel = this._getFromInput();
    } else {
      sel = this._getFromDoc();
    }

    if (sel === null) {
      util.log("sel: null");
      return null;
    }

    util.log("sel:"+sel);
    if (sel.indexOf("\n") > - 1) {
      return null;
    }
    if (sel !== sel.trim()) {
      return null;
    }
    return sel;
  }
};



function findClickedWord(chromeDoc) {
  var sel = new Selection(chromeDoc);
  if (sel.hasSelection) {
    var s = sel.getValidSelection();
    if (s !== null) {
      return s;
    }
    return null;
  }

  // code valid for document or input

  // if empty=NaN; empty input|empty region textatra=node.data=undefined;
  var pos = chromeDoc.popupRangeOffset;
  var txtNode = isNaN(pos) || chromeDoc.popupRangeParent === null ? undefined : chromeDoc.popupRangeParent.data;
  if (txtNode === undefined) {
    util.log('no txt,' + chromeDoc.popupRangeParent + pos);
    return null;
  }

  var wordIdx = util.findCurrentUrlPos(txtNode, pos);
  return txtNode.substr(wordIdx[0], wordIdx[1]);
}



function isTextInputNode(clickedNode) {
  if (!clickedNode) {
    return false;
  }
  //document.designMode === 'on'
  switch (clickedNode.tagName) { // localName
    case "INPUT":
    case "html:input":
      return clickedNode.type === 'text';
    case "TEXTAREA":
    case "html:textarea":
      return true;
    default:
      return false;
  }
}


function isTextMenu(menu) {
  var items = menu.getElementsByTagNameNS(NS_XUL, "menuitem");
  var ok = false;
  for (var idx = 0, qty = items.length; idx < qty; idx++) {
    var item = items[idx];

    var id = "";
    if (item.hasAttribute("command")) {
      id = item.getAttribute("command");
    } else if (item.hasAttribute("cmd")) {
      id = item.getAttribute("cmd");
    } else {
      continue;
    }

    if (id === "cmd_find") {
      return false;
    }
    if (id === "cmd_cut") {
      ok = true;
    }
  }
  return ok;
}

function toUri(txt) {
  var uri = null;
  if (!txt.indexOf) {
    //util.log("toUri " + txt + (typeof txt));
    util.myThrow(txt);
  }
  if (txt.indexOf("www.") === 0) {
    uri = util.stringToUri("http://" + txt);
  } else {
    uri = util.stringToUri(txt);
  }

  if (uri === null) {
    return null;
  }

  switch (uri.scheme) {
    case "http":
    case "https":
    case "ftp":
    case "mailto":
      return uri;
  }
  return null;
}
