/*
 * Copyright (C) 2009 Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

const EXPORTED_SYMBOLS = ["redirStorage"];

const ORGANIZER_QUERY_ANNO = "PlacesOrganizer/OrganizerQuery";
const ANNOTATION_LEFTPANE_VALUE = "hultmann/migre.me/organizer";
const ANNOTATION_REDIR_NAME = "hultmann/migre.me/urls";


Components.utils.import("resource://hultmann-migre-me-modules/util.js");

function service() {
  return Cc["@mozilla.org/browser/annotation-service;1"].getService(Ci.nsIAnnotationService);
}


const redirStorage = {

  createOrganizerFolder: function(winOrganizer) {

    var as = Cc["@mozilla.org/browser/annotation-service;1"].getService(Ci.nsIAnnotationService);
    var bs = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Ci.nsINavBookmarksService);

    var itemId = bs.createFolder(winOrganizer.PlacesUIUtils.leftPaneFolderId, "migre.me", bs.DEFAULT_INDEX);
    util.log("createOrganizerFolder=" + itemId);
    as.setItemAnnotation(itemId,
                         ORGANIZER_QUERY_ANNO, ANNOTATION_LEFTPANE_VALUE,
                         0, as.EXPIRE_NEVER);
    as.setItemAnnotation(itemId,
                         winOrganizer.EXCLUDE_FROM_BACKUP_ANNO, 1,
                         0, as.EXPIRE_NEVER);
    return itemId;
  },


  removeOrganizerItem: function() {
    var id = this.getOrganizerItemId();
    if (id !== null) {
      Cc["@mozilla.org/browser/nav-bookmarks-service;1"]
        .getService(Ci.nsINavBookmarksService)
        .removeItem(id);
    }
  },


  getOrganizerItemId: function() {
    var as = Cc["@mozilla.org/browser/annotation-service;1"].getService(Ci.nsIAnnotationService);
    var items = as.getItemsWithAnnotation(ORGANIZER_QUERY_ANNO, {});

    for (var idx = 0; idx < items.length; idx++) {
      var val = as.getItemAnnotation(items[idx], ORGANIZER_QUERY_ANNO);
      if (val === ANNOTATION_LEFTPANE_VALUE) {
        return items[idx];
      }
    }
    return null;
  },


  saveShortUrl: function(longUri, shortUri) {

    if (Cc["@mozilla.org/browser/global-history;2"]
        .getService(Ci.nsIGlobalHistory2)
        .isVisited(longUri) === false) {

      var nhs = Cc["@mozilla.org/browser/nav-history-service;1"]
                  .getService(Ci.nsINavHistoryService);
      //long addVisit
      nhs.addVisit(longUri,             // nsIURI  aURI
                   Date.now() * 1000,   // PRTime  aTime
                   null,                // long    aReferringVisit
                   nhs.TRANSITION_LINK, // long    aTransitionType
                   false,               // boolean aIsRedirect
                   0);                  // long    aSessionID
      util.log("addVisit " + longUri.spec);
    }

    var shortUrl = shortUri.spec;
    var shortUrls = this.loadShortUrls(longUri);

    if (shortUrls.hasOwnProperty(shortUrl)) {
      util.log("saveShortUrl skip " + shortUrl);
      return;
    }

    shortUrls[shortUrl] = {
      //"clicks-api":
      //"htmlTitle-api"
      //"created-api"
      //contentType
      "dateAdded": new Date()   //{added:"2009-10-27T00:51:17.604Z"}
    };

    var serv = service();
    serv.setPageAnnotation(longUri,
                           ANNOTATION_REDIR_NAME,
                           JSON.stringify(shortUrls),
                           0,
                           serv.EXPIRE_WITH_HISTORY);
  },


  removeShortUrl: function(longUri, shortUri) {
    var shortUrl = shortUri.spec;
    var shortUrls = this.loadShortUrls(longUri);

    if (shortUrls.hasOwnProperty(shortUrl) === false) {
      util.log("removeShortUrl skip " + shortUrl);
      return false;
    }

    util.log("removeShortUrl before " + shortUrls.toSource());
    delete shortUrls[shortUrl];

    var serv = service();
    if (shortUrls.__count__ > 0) {
      util.log("removeShortUrl after " + shortUrls.toSource());
      serv.setPageAnnotation(longUri,
                             ANNOTATION_REDIR_NAME,
                             JSON.stringify(shortUrls),
                             0,
                             serv.EXPIRE_WITH_HISTORY);
    } else {
      serv.removePageAnnotation(longUri, ANNOTATION_REDIR_NAME);
      util.log("removeShortUrl after removed!");
    }
    return true;
  },


  getSavedRedir: function(longUri) {
    var json = this.loadShortUrls(longUri);
    for (var shortUrl in json) {
      if (json.hasOwnProperty(shortUrl)) {
        return shortUrl;
      }
    }
    return null;
  },


  loadShortUrls: function(longUri) {
    var serv = service();
    if (serv.pageHasAnnotation(longUri, ANNOTATION_REDIR_NAME) === false) {
      return {};
    }

    var a = serv.getPageAnnotation(longUri, ANNOTATION_REDIR_NAME);
    try {
      return JSON.parse(a);
    } catch (ex) {
      util.log("loadShortUrls: json annotation fail " + a);
      return {};
    }
  },

  getPagesWithShortUrl: function() {
    return service().getPagesWithAnnotation(ANNOTATION_REDIR_NAME, {});
  }

};
