/*
 * Copyright (C) 2009 Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

const EXPORTED_SYMBOLS = ["ResolveRedir"];

Components.utils.import("resource://hultmann-migre-me-modules/util.js");


function ResolveRedir() {
  this.onError = null;
  this.onSuccess = null;
  this.onComplete = null;
}


ResolveRedir.prototype = {

  findRedir: function(uri) {
    var xhr = this._createXhr(uri);
    xhr.channel.QueryInterface(Ci.nsIHttpChannel).redirectionLimit = 0;
    var me = this;
    xhr.onload = function(evt) {
      me.onComplete();
      me._load(this);  // not a HTTP redir
    }
    xhr.onerror = function(evt) {
      me.onComplete();
      me._error(this); // redir=error
    }
    xhr.send(null);
  },


  _createXhr: function(uri) {
    var xhr = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);

    xhr.open("HEAD", uri.spec, true);
    xhr.setRequestHeader("Accept-Language", null);
    xhr.setRequestHeader("Cookie", null);
    xhr.setRequestHeader("Accept", null);
    xhr.setRequestHeader("User-Agent", "SubLink/3.0.0 (+http://br.mozdev.org/migre.me/)");
    // LOAD_ONLY_IF_MODIFIED makes xhr results from cache be empty!
    //xhr.channel.loadFlags |= Ci.nsICachingChannel.LOAD_ONLY_IF_MODIFIED;
    return xhr;
  },


  _getRefresh: function(xhr) {
    var header = xhr.getResponseHeader("Refresh");
    if (header === null) {
      return null;
    }
    var fields = header.split(";", 2);       // ["0 ", " url=http "]
    if (fields.length !== 2) {
      return null;
    }

    var nameValue = fields[1].split("=", 2); // [" url", "http "]
    if (nameValue.length !== 2) {
      return null;
    }

    if (nameValue[0].trim().toLowerCase() !== "url") {
      return null;
    }

    return util.stringToUri(nameValue[1].trim());
  },


  _load: function(xhr2) {

    switch (xhr2.status) {
      case 200:
        var uri = this._getRefresh(xhr2);
        if (uri) {
          this.onSuccess({
            longUri:            uri,
            responseStatusCode: xhr2.status
          });
          return;
        }
        break;
    }


    util.log(xhr2.status + ' - ' + xhr2.statusText);
    if (xhr2.status > 400) {
      this.onError({
        id: "couldntrestore", // server-error?
        xhr: xhr2,
        msg: xhr2.status + ' - ' + xhr2.statusText
      });
    } else {
      this.onError({
        id: "couldntrestore-short",
        xhr: xhr2,
        msg: null
      });
      util.log("response:\n" + xhr2.responseText);
    }
  },


  _error: function(xhr2) {

    switch (xhr2.status) {
      case 301:
      case 302:
        break;
      default: // 0 = no response
        this.onError({
          id: "noconnection",
          xhr: xhr2,
          msg: null
        });
        return;
    }

    // status ok

    var uri = resolveUrl(xhr2.getResponseHeader("Location"), xhr2.channel.originalURI);
    if (uri === null) {
      util.log("Location: " + uri.spec);
      this.onError({
        id: "invalid",
        xhr: xhr2, // BUG xhr will be empty
        msg: null
      });
      return;
    }

    this.onSuccess({
      longUri:            uri,
      responseStatusCode: xhr2.status
    });
  }

};

function resolveUrl(url, uri) {
  if (url === null) {
    return null;
  }
  if (url.indexOf("/") === 0) {
    url = uri.resolve(url);
  }
  return util.stringToUri(url);
}
